@@ -80,9 +80,14 @@ def fmt_short_comment_body(body):
         and not line.startswith('#')  # Markdown heading
         and not line.startswith('<!-')  # commented out HTML-style
     ]
-    short = textwrap.wrap(lines[0], 250)[0]
-    if len(lines) > 1 or short != lines[0]:
+    # abbreviate commit hashes in the text
+    line = re.sub(r'[a-f0-9]{40}', lambda m: m.group(0)[:7], lines[0])
+    # wrap text to get a line of at most 250 chars
+    short = textwrap.wrap(line, 250)[0]
+    # add continuation marker if needed
+    if len(lines) > 1 or short != line:
         short += ' […]'
+
     return short
 
 
@@ -183,6 +188,18 @@ def get_issue_or_pr_number(payload=None):
     return number
 
 
+def get_issue_or_pr_title(payload=None):
+    if not payload:
+        payload = current_payload
+
+    try:
+        title = payload['issue']['title']
+    except KeyError:
+        title = payload['pull_request']['title']
+
+    return title
+
+
 def fmt_push_summary_message(payload=None, row=None):
     if not payload:
         payload = current_payload
@@ -263,6 +280,37 @@ def fmt_issue_summary_message(payload=None):
                   emojize(payload['issue']['title']))
 
 
+def fmt_issue_incoming_transfer_message(payload=None):
+    if not payload:
+        payload = current_payload
+    # GitHub unfortunately doesn't seem to include any info about the user who
+    # initiated the issue transfer, only the original author/creator.
+    return '[{}] {}#{} by {} was transferred to issue #{}: {}'.format(
+                  fmt_repo(payload['repository']['name']),
+                  fmt_repo(payload['changes']['old_repository']['full_name']),
+                  payload['changes']['old_issue']['number'],
+                  fmt_name(payload['issue']['user']['login']),
+                  payload['issue']['number'],
+                  emojize(payload['issue']['title']))
+
+
+def fmt_issue_outgoing_transfer_message(payload=None):
+    if not payload:
+        payload = current_payload
+    # For "transferred" events (sent for the source repo only), GitHub DOES set
+    # the "sender" info to the user who initiated the transfer, unlike for
+    # inbound transfer hooks (which just look like any other "opened" event
+    # except for the addition of a "changes" object).
+    return '[{}] {} transferred issue #{} by {} to {}#{}: {}'.format(
+                  fmt_repo(payload['repository']['name']),
+                  fmt_name(payload['sender']['login']),
+                  payload['issue']['number'],
+                  fmt_name(payload['issue']['user']['login']),
+                  fmt_repo(payload['changes']['new_repository']['full_name']),
+                  payload['changes']['new_issue']['number'],
+                  emojize(payload['issue']['title']))
+
+
 def fmt_issue_title_edit(payload=None):
     if not payload:
         payload = current_payload
@@ -277,35 +325,40 @@ def fmt_issue_title_edit(payload=None):
 def fmt_issue_assignee_message(payload=None):
     if not payload:
         payload = current_payload
-    
+
     target = ''
+    assignee = payload['assignee']['login']
     self_assign = False
-    if (payload['assignee']['login'] == payload['sender']['login']):
+
+    if assignee == payload['sender']['login']:
         self_assign = True
     else:
-        target = 'to ' if payload['action'] == 'assigned' else 'from '
-        target = target + fmt_name(payload['assignee']['login']) 
-    return '[{}] {} {}{} {} #{} {}'.format(
+        prep = 'to' if payload['action'] == 'assigned' else 'from'
+        target = ' {} {}'.format(prep, fmt_name(assignee))
+
+    return '[{}] {} {}{} {} #{}{} ({})'.format(
                   fmt_repo(payload['repository']['name']),
                   fmt_name(payload['sender']['login']),
                   'self-' if self_assign else '',
                   payload['action'],
                   get_issue_type(payload),
                   get_issue_or_pr_number(payload),
-                  target)
+                  target,
+                  get_issue_or_pr_title(payload))
 
 
 def fmt_issue_label_message(payload=None):
     if not payload:
         payload = current_payload
-    return '[{}] {} {} the label \'{}\' {} {} #{}'.format(
+    return '[{}] {} {} the label \'{}\' {} {} #{} ({})'.format(
                   fmt_repo(payload['repository']['name']),
                   fmt_name(payload['sender']['login']),
                   'added' if payload['action'] == 'labeled' else 'removed',
                   payload['label']['name'],
                   'to' if payload['action'] == 'labeled' else 'from',
                   get_issue_type(payload),
-                  get_issue_or_pr_number(payload))
+                  get_issue_or_pr_number(payload),
+                  get_issue_or_pr_title(payload))
 
 
 def fmt_issue_milestone_message(payload=None):
@@ -314,12 +367,13 @@ def fmt_issue_milestone_message(payload=None):
 
     added = payload['action'] == 'milestoned'
 
-    return '[{}] {} {} {} #{} {} the {} milestone'.format(
+    return '[{}] {} {} {} #{} ({}) {} the {} milestone'.format(
                   fmt_repo(payload['repository']['name']),
                   fmt_name(payload['sender']['login']),
                   'added' if added else 'removed',
                   get_issue_type(payload),
                   get_issue_or_pr_number(payload),
+                  get_issue_or_pr_title(payload),
                   'to' if added else 'from',
                   payload['milestone']['title'])
 
@@ -342,9 +396,6 @@ def fmt_pull_request_summary_message(payload=None):
     if not payload:
         payload = current_payload
 
-    base_ref = payload['pull_request']['base']['label'].split(':')[-1]
-    head_ref = payload['pull_request']['head']['label'].split(':')[-1]
-
     action = payload['action']
     if action == 'closed' and payload['pull_request']['merged']:
         action = 'merged'
@@ -352,15 +403,32 @@ def fmt_pull_request_summary_message(payload=None):
         action = 'drafted'
     elif action == 'ready_for_review':
         action = 'readied'
+    elif action == 'converted_to_draft':
+        action = 'un-readied'
 
-    return '[{}] {} {} pull request #{}: {} ({}...{})'.format(
+    actor = payload['sender']['login']
+    author = payload['pull_request']['user']['login']
+    maybe_possessive = ''
+    if action == 'merged' and actor != author:
+        maybe_possessive = '%s\'s ' % fmt_name(author)
+
+    base = fmt_branch(payload['pull_request']['base']['ref'])
+    head = fmt_branch(payload['pull_request']['head']['ref'])
+    base_repo = payload['pull_request']['base']['user']['login']
+    head_repo = payload['pull_request']['head']['user']['login']
+    if base_repo != head_repo:
+        base = "{}:{}".format(fmt_name(base_repo), base)
+        head = "{}:{}".format(fmt_name(head_repo), head)
+
+    return '[{}] {} {} {}pull request #{}: {} ({}...{})'.format(
                   fmt_repo(payload['repository']['name']),
-                  fmt_name(payload['sender']['login']),
+                  fmt_name(actor),
                   action,
+                  maybe_possessive,
                   payload['pull_request']['number'],
                   emojize(payload['pull_request']['title']),
-                  fmt_branch(base_ref),
-                  fmt_branch(head_ref))
+                  base,
+                  head)
 
 
 def fmt_pull_request_title_edit(payload=None):
@@ -518,7 +586,7 @@ def get_formatted_response(payload, row):
     elif payload['event'] == 'commit_comment':
         messages.append(fmt_commit_comment_summary() + " " + fmt_url(shorten_url(payload['comment']['html_url'])))
     elif payload['event'] == 'pull_request':
-        if re.match('((re)?open|clos)ed', payload['action']) or payload['action'] == 'ready_for_review':
+        if re.match('((re)?open|clos)ed', payload['action']) or payload['action'] in ['ready_for_review', 'converted_to_draft']:
             messages.append(fmt_pull_request_summary_message() + " " + fmt_url(shorten_url(payload['pull_request']['html_url'])))
         elif payload['action'] == 'edited':
             if 'changes' in payload:
@@ -527,7 +595,10 @@ def get_formatted_response(payload, row):
         elif re.match('(assigned|unassigned)', payload['action']):
             messages.append(fmt_issue_assignee_message() + " " + fmt_url(shorten_url(payload['pull_request']['html_url'])))
         elif re.match('(labeled|unlabeled)', payload['action']):
-            messages.append(fmt_issue_label_message() + " " + fmt_url(shorten_url(payload['pull_request']['html_url'])))
+            if payload.get('label', None):
+                # If a label is deleted, for example, we'll get a webhook payload with no details about the removed label.
+                # We skip those; there's no reason to emit action messages to IRC with "unknown label" placeholders.
+                messages.append(fmt_issue_label_message() + " " + fmt_url(shorten_url(payload['pull_request']['html_url'])))
     elif payload['event'] == 'pull_request_review':
         if payload['action'] == 'submitted' and payload['review']['state'] in ['approved', 'changes_requested', 'commented']:
             if payload['review']['state'] == 'commented' and payload['review']['body'] == None:
@@ -543,17 +614,25 @@ def get_formatted_response(payload, row):
         messages.append(fmt_pull_request_review_comment_summary_message() + " " + fmt_url(shorten_url(payload['comment']['html_url'])))
     elif payload['event'] == 'issues':
         if re.match('((re)?open|clos)ed', payload['action']):
-            messages.append(fmt_issue_summary_message() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
+            if 'changes' in payload and all(k in payload['changes'] for k in ['old_repository', 'old_issue']):
+                messages.append(fmt_issue_incoming_transfer_message() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
+            else:
+                messages.append(fmt_issue_summary_message() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
         elif re.match('(assigned|unassigned)', payload['action']):
             messages.append(fmt_issue_assignee_message() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
         elif re.match('(labeled|unlabeled)', payload['action']):
-            messages.append(fmt_issue_label_message() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
+            if payload.get('label', None):
+                # If a label is deleted, for example, we'll get a webhook payload with no details about the removed label.
+                # We skip those; there's no reason to emit action messages to IRC with "unknown label" placeholders.
+                messages.append(fmt_issue_label_message() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
         elif re.match('(milestoned|demilestoned)', payload['action']):
             messages.append(fmt_issue_milestone_message() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
         elif payload['action'] == 'edited':
             if 'changes' in payload:
                 if 'title' in payload['changes']:
                     messages.append(fmt_issue_title_edit() + " " + fmt_url(shorten_url(payload['issue']['html_url'])))
+        elif payload['action'] == 'transferred':
+            messages.append(fmt_issue_outgoing_transfer_message() + " " + fmt_url(shorten_url(payload['changes']['new_issue']['html_url'])))
     elif payload['event'] == 'issue_comment' and payload['action'] == 'created':
         messages.append(fmt_issue_comment_summary_message() + " " + fmt_url(shorten_url(payload['comment']['html_url'])))
     elif payload['event'] == 'gollum':
