@@ -13,8 +13,8 @@ Copyright 2019 dgw
 
 from __future__ import unicode_literals
 from sopel import tools
-from sopel.module import OP, NOLIMIT, commands, example, require_chanmsg, url
-from sopel.formatting import bold, color
+from sopel.module import OP, NOLIMIT, commands, example, require_chanmsg, rule, url
+from sopel.formatting import bold, color, monospace
 from sopel.tools.time import get_timezone, format_time
 from sopel.config.types import StaticSection, ValidatedAttribute
 
@@ -36,6 +36,7 @@ import json
 import requests
 import re
 import datetime
+import base64
 
 '''
  _______           __         __
@@ -53,10 +54,11 @@ githubUsername = r'[A-Za-z\d](?:[A-Za-z\d]|-(?=[A-Za-z\d])){0,38}'
 # not copied from anywhere, but handy to simply reuse
 githubRepoSlug = r'[A-Za-z0-9\.\-]+'
 # lots of regex and other globals to make this stuff work
-baseURL = r'https?://(?:www\.)?github.com/({username}/{repo})'.format(username=githubUsername, repo=githubRepoSlug)
+baseURL = r'https?://(?:www\.)?github\.com/({username}/{repo})'.format(username=githubUsername, repo=githubRepoSlug)
 repoURL = baseURL + r'/?(?!\S)'
 issueURL = baseURL + r'/(?:issues|pull)/([\d]+)(?:#issuecomment-([\d]+))?'
 commitURL = baseURL + r'/(?:commit)/([A-z0-9\-]+)'
+contentURL = baseURL + r'/(?:blob|raw)/([^/\s]+)/([^#\s]+)(?:#L(\d+)(?:-L(\d+))?)?'
 
 
 class GitHubSection(StaticSection):
@@ -117,12 +119,29 @@ def fetch_api_endpoint(bot, url):
     return requests.get(url, auth=auth).text
 
 
+@rule(r'.*(?<!\S)/?#(\d+)\b.*')
+@require_chanmsg
+def issue_reference(bot, trigger):
+    """
+    Separate function to work around Sopel not loading rules/commands for @url callables.
+    """
+    issue_info(bot, trigger)
+
+
 @url(issueURL)
 def issue_info(bot, trigger, match=None):
-    match = match or trigger
-    URL = 'https://api.github.com/repos/%s/issues/%s' % (match.group(1), match.group(2))
-    if (match.group(3)):
-        URL = 'https://api.github.com/repos/%s/issues/comments/%s' % (match.group(1), match.group(3))
+    if match:  # Link triggered
+        repo = match.group(1)
+        num = match.group(2)
+        URL = 'https://api.github.com/repos/%s/issues/%s' % (repo, num)
+        if (match.group(3)):
+            URL = 'https://api.github.com/repos/%s/issues/comments/%s' % (repo, match.group(3))
+    else:  # Issue/PR number triggered
+        repo = bot.db.get_channel_value('github_issue_repo', trigger.sender)
+        num = trigger.group(1)
+        if not repo:
+            return NOLIMIT
+        URL = 'https://api.github.com/repos/%s/issues/%s' % (repo, num)
 
     try:
         raw = fetch_api_endpoint(bot, URL)
@@ -133,7 +152,7 @@ def issue_info(bot, trigger, match=None):
     try:
         body = data['body']
     except (KeyError):
-        bot.say('[GitHub] API says this is an invalid issue. Please report this if you know it\'s a correct link!')
+        bot.say('[GitHub] API says this is an invalid issue. Please report this if you know it should work!')
         return NOLIMIT
 
     if body.strip() == '':
@@ -144,9 +163,9 @@ def issue_info(bot, trigger, match=None):
     response = [
         bold('[GitHub]'),
         ' [',
-        match.group(1),
+        repo,
         ' #',
-        match.group(2),
+        num,
         '] ',
         data['user']['login'],
         ': '
@@ -157,9 +176,33 @@ def issue_info(bot, trigger, match=None):
         response.append(bold(' | '))
     response.append(emojize(body))
 
+    # append link, if not triggered by a link
+    if not match:
+        link = shorten_url(data['html_url'])
+        response.append(bold(' | '))
+        response.append(link)
+
     bot.say(''.join(response))
 
 
+@commands('gh-repo')
+@example('.gh-repo sopel-irc/sopel-github')
+@require_chanmsg('[GitHub] You can only link a repository to a channel.')
+def manage_channel_repo(bot, trigger):
+    """
+    Set the repository to use for looking up standalone issue/PR references.
+    """
+    allowed = bot.channels[trigger.sender].privileges.get(trigger.nick, 0) >= OP
+    if not allowed and not trigger.admin:
+        return bot.say('You must be a channel operator to use this command!')
+
+    if not trigger.group(2):
+        return bot.say(bot.db.get_channel_value('github_issue_repo', trigger.sender))
+
+    bot.db.set_channel_value('github_issue_repo', trigger.sender, trigger.group(3))
+    bot.reply('Set linked repo for %s to %s.' % (trigger.sender, trigger.group(3)))
+
+
 @url(commitURL)
 def commit_info(bot, trigger, match=None):
     match = match or trigger
@@ -204,6 +247,58 @@ def commit_info(bot, trigger, match=None):
     bot.say(''.join(response))
 
 
+@url(contentURL)
+def file_info(bot, trigger, match=None):
+    match = match or trigger
+    repo = match.group(1)
+    path = match.group(3)
+    ref = match.group(2)
+    start_line = match.group(4)
+    end_line = match.group(5)
+    URL = 'https://api.github.com/repos/%s/contents/%s?ref=%s' % (repo, path, ref)
+
+    try:
+        raw = fetch_api_endpoint(bot, URL)
+    except HTTPError:
+        bot.say('[GitHub] API returned an error.')
+        return NOLIMIT
+    data = json.loads(raw)
+
+    if data.get('type', 'dir') != 'file':
+        # silently ignore directory contents (and malformed responses) for now
+        return NOLIMIT
+
+    response = [
+        bold('[GitHub]'),
+        ' [',
+        repo,
+        '] ',
+        data['path'],
+        ' @ ',
+        ref,
+    ]
+
+    if start_line:
+        lines = base64.b64decode(data['content']).splitlines()
+
+        try:
+            snippet = lines[int(start_line) - 1].decode('utf-8')
+        except (IndexError, UnicodeDecodeError):
+            # Line doesn't exist, or not a text file
+            snippet = None
+
+        if snippet:
+            response.extend([
+                ' | L',
+                start_line,
+                ': ',
+                monospace(snippet),
+                ' […] (to L%s)' % end_line if end_line else '',
+            ])
+
+    bot.say(''.join(response))
+
+
 def get_data(bot, trigger, URL):
     URL = URL.split('#')[0]
     try:
@@ -248,8 +343,12 @@ def repo_info(bot, trigger, match=None):
 
 
 @commands('github', 'gh')
+@example('.gh sopel-irc/sopel-github')
 def github_repo(bot, trigger):
-    repo = trigger.group(2) or trigger.group(1)
+    repo = trigger.group(3) or None
+
+    if repo is None:
+        return bot.reply('I need a repository name, or `user/reponame`.')
 
     if repo.lower() == 'version':
         return bot.say('[GitHub] Version {} by {}, report issues at {}'.format(
